@@ -1,7 +1,7 @@
 /*
     xattr - a python module for manipulating filesystem extended attributes
 
-    Copyright (C) 2002, 2003, 2006, 2008, 2012 Iustin Pop <iusty@k1024.org>
+    Copyright (C) 2002, 2003, 2006, 2008, 2012, 2013 Iustin Pop <iusty@k1024.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -147,7 +147,13 @@ static int convert_obj(PyObject *myobj, target_t *tgt, int nofollow) {
         tgt->type = nofollow ? T_LINK : T_PATH;
         tgt->tmp = \
             PyUnicode_AsEncodedString(myobj,
-                                      Py_FileSystemDefaultEncoding, "strict");
+                                      Py_FileSystemDefaultEncoding,
+#ifdef IS_PY3K
+                                      "surrogateescape"
+#else
+                                      "strict"
+#endif
+                                      );
         if(tgt->tmp == NULL)
             return -1;
         tgt->name = PyBytes_AS_STRING(tgt->tmp);
@@ -467,6 +473,11 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
         goto freetgt;
     }
 
+    if(nalloc == 0) {
+        res = PyList_New(0);
+        goto freetgt;
+    }
+
     /* Try to allocate the memory, using Python's allocator */
     if((buf_list = PyMem_Malloc(nalloc)) == NULL) {
         res = PyErr_NoMemory();
@@ -593,7 +604,7 @@ pysetxattr(PyObject *self, PyObject *args)
     int nofollow = 0;
     char *attrname = NULL;
     char *buf = NULL;
-    Py_ssize_t bufsize;
+    int bufsize;
     int nret;
     int flags = 0;
     target_t tgt;
@@ -659,7 +670,7 @@ xattr_set(PyObject *self, PyObject *args, PyObject *keywds)
     int nofollow = 0;
     char *attrname = NULL;
     char *buf = NULL;
-    Py_ssize_t bufsize;
+    int bufsize;
     int nret;
     int flags = 0;
     target_t tgt;
@@ -864,6 +875,11 @@ pylistxattr(PyObject *self, PyObject *args)
         goto freetgt;
     }
 
+    if(nalloc == 0) {
+        mylist = PyList_New(0);
+        goto freetgt;
+    }
+
     /* Try to allocate the memory, using Python's allocator */
     if((buf = PyMem_Malloc(nalloc)) == NULL) {
         mylist = PyErr_NoMemory();
@@ -964,6 +980,11 @@ xattr_list(PyObject *self, PyObject *args, PyObject *keywds)
         goto freetgt;
     }
 
+    if(nalloc == 0) {
+        res = PyList_New(0);
+        goto freetgt;
+    }
+
     /* Try to allocate the memory, using Python's allocator */
     if((buf = PyMem_Malloc(nalloc)) == NULL) {
         res = PyErr_NoMemory();
